// Requality: CKEditor plugin
// Implements functions used by the UniEditor component of the Requality project
// Details:
//   1. New toolbar button RequalitySave that saves changes and closes the editor.
//   2. New toolbar button RequalityExit that closes the editor without saving
//      changes (confirmation message is shown if changes are present).

// TODO: Calling requalitySave/requalityExit causes JS errors. Probably, editor
// instance is destroyed too early. Find out how to work around the problem!

(function () {
	// Command implementing RequalitySave action
	var saveCmd = {
		exec: function (editor) {
			editor.destroy(false);
		}
	};
	// Command implementing RequalityExit action
	var exitCmd = {
		exec: function (editor) {
			if (editor.checkDirty())
				if (!confirm('Имеются несохранённые изменения. Подтверждаете их отмену?')) {
					return;
				}
			editor.destroy(true);
		}
	};

	// Register the new plugin named "requality".
	CKEDITOR.plugins.add('requality', {
		init: function (editor) {
			editor.addCommand('requalitySave', saveCmd);
			editor.addCommand('requalityExit', exitCmd);
			editor.ui.addButton('RequalitySave', {
				label: 'Сохранить изменения и закрыть редактор',
				command: 'requalitySave'
			});
			editor.ui.addButton('RequalityExit', {
				label: 'Закрыть редактор без сохранения изменений',
				command: 'requalityExit'
			});
		}
	});
})();
