package com.unitesk.requality.core;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.UUID;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

public class NodeMapper {
    private String nodeString;

    public NodeMapper(String string) {
        nodeString = string;
    }

    public UUID getUUId() {
        if (nodeString == null) {
            return null;
        }
        try {
            JSONObject obj = new JSONObject(nodeString);
            return UUID.fromString(obj.getString("uuid"));
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return null;
    }

    public Collection<Attribute> getAttributes() {
        if (nodeString == null) {
            return null;
        }
        Collection<Attribute> result = new HashSet<Attribute>();
        try {
            JSONObject obj = new JSONObject(nodeString);
            JSONArray attr = obj.getJSONArray("attributes");
            for (int i = 0; i < attr.length(); i++) {
                JSONObject a = attr.getJSONObject(i);
                AttributeType type = AttributeType.valueOf(a.getString("type"));
                String key = a.getString("key");
                Object value = null;
                if (type == AttributeType.LIST) {
                    ArrayList<String> attrLst = new ArrayList<String>();
                    JSONArray vals = a.getJSONArray("value");
                    for (int j = 0; j < vals.length(); j++) {
                        attrLst.add(vals.getString(j));
                    }
                    value = attrLst;
                } else {
                    // TODO: all attribute types support
                    value = type.fromString(a.getString("value"));
                }
                result.add(new Attribute(type, key, value));
            }
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return result;
    }

    @SuppressWarnings("unchecked")
    public final static JSONObject toJSON(TreeNode node) {
        JSONObject result = null;
        try {
            result = new JSONObject();
            result.put("uuid", node.getUUId());
            JSONArray attrs = new JSONArray();
            for (Attribute a : node.attributes.values()) {
                JSONObject attrObj = new JSONObject();
                attrObj.put("key", a.getKey());
                attrObj.put("type", a.getType());
                if (a.getType() != AttributeType.LIST) {
                    attrObj.put("value", a.getValue());
                } else {
                    JSONArray lst = new JSONArray();
                    List<String> al = (List<String>) a.getValue();
                    for (String i : al) {
                        lst.put(i);
                    }
                    attrObj.put("value", lst);
                }
                attrs.put(attrObj);
            }
            result.put("attributes", attrs);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return result;
    }

    @SuppressWarnings("unchecked")
    public final static JSONObject toJSON(UUID uuid, Collection<Attribute> attr) {
        JSONObject result = null;
        try {
            result = new JSONObject();
            result.put("uuid", uuid);
            JSONArray attrs = new JSONArray();
            for (Attribute a : attr) {
                JSONObject attrObj = new JSONObject();
                attrObj.put("key", a.getKey());
                attrObj.put("type", a.getType());
                if (a.getType() != AttributeType.LIST) {
                    attrObj.put("value", a.getValue());
                } else {
                    JSONArray lst = new JSONArray();
                    List<String> al = (List<String>) a.getValue();
                    for (String i : al) {
                        lst.put(i);
                    }
                    attrObj.put("value", lst);
                }
                attrs.put(attrObj);
            }
            result.put("attributes", attrs);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return result;
    }

}
