/*
 * Copyright 2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.editors;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;

import com.unitesk.requality.core.TreeNode;

public class AdditionalPageReqEditor {
   TreeNode req= null;
   
   private Composite parent= null;
   private Composite top= null;
   
   private Composite topComposite= null;
   private Composite bottomComposite= null;
   
   private Group leftGroup= null;
   private Group rightGroup= null;
   private Group bottomGroup= null;
   
   private Composite paramsComposite= null;
   private Button addParamButton= null;
   private Button removeParamButton= null;
      
   private Composite attrsComposite= null;
   private Button addAttrButton= null;
   private Button removeAttrButton= null;
   private Button editParamButton= null;
   private Button editAttrButton= null;
   
   public AdditionalPageReqEditor( Composite parent, TreeNode req ) {
      this.req= req;
      this.parent= parent;
   }
   
   public Composite createPage() {
      createPartControl( parent );
      return top;
   }
   
   public void createPartControl( Composite parent ) {
      FillLayout fillLayout= new FillLayout();
      fillLayout.type= org.eclipse.swt.SWT.VERTICAL;
      top= new Composite( parent, SWT.NONE );
      createTopComposite();
      top.setLayout( fillLayout );
      createBottomComposite();
   }
   
   /**
    * This method initializes topComposite
    * 
    */
   private void createTopComposite() {
      topComposite= new Composite( top, SWT.NONE );
      createLeftGroup();
      topComposite.setLayout( new FillLayout() );
      createRightGroup();
   }
   
   /**
    * This method initializes bottomComposite
    * 
    */
   private void createBottomComposite() {
      FillLayout fillLayout1= new FillLayout();
      fillLayout1.marginHeight= 0;
      bottomComposite= new Composite( top, SWT.NONE );
      createBottomGroup();
      bottomComposite.setLayout( fillLayout1 );
   }
   
   /**
    * This method initializes leftGroup
    * 
    */
   private void createLeftGroup() {
      leftGroup= new Group( topComposite, SWT.NONE );
      leftGroup.setLayout( new FillLayout() );
      leftGroup.setText( "Parameters" );
      // paramsTree =
      new Tree( leftGroup, SWT.NONE );
      createParamsComposite();
   }
   
   /**
    * This method initializes rightGroup
    * 
    */
   private void createRightGroup() {
      rightGroup= new Group( topComposite, SWT.NONE );
      rightGroup.setText( "Attributes" );
      rightGroup.setLayout( new FillLayout() );
      // attrsTree =
      new Tree( rightGroup, SWT.NONE );
      createAttrsComposite();
   }
   
   /**
    * This method initializes bottomGroup
    * 
    */
   private void createBottomGroup() {
      bottomGroup= new Group( bottomComposite, SWT.NONE );
      bottomGroup.setLayout( new FillLayout() );
      bottomGroup.setText( "Predicate" );
      // predicateArea =
      new Text( bottomGroup, SWT.MULTI | SWT.WRAP | SWT.V_SCROLL );
   }
   
   /**
    * This method initializes paramsComposite
    * 
    */
   private void createParamsComposite() {
      GridData gridData4= new GridData();
      gridData4.widthHint= 50;
      GridLayout gridLayout= new GridLayout();
      gridLayout.numColumns= 1;
      GridData gridData1= new GridData();
      gridData1.widthHint= 50;
      GridData gridData= new GridData();
      gridData.widthHint= 50;
      paramsComposite= new Composite( leftGroup, SWT.BORDER );
      paramsComposite.setLayout( gridLayout );
      addParamButton= new Button( paramsComposite, SWT.NONE );
      addParamButton.setText( "Add" );
      addParamButton.setLayoutData( gridData );
      editParamButton= new Button( paramsComposite, SWT.NONE );
      editParamButton.setText( "Edit" );
      editParamButton.setLayoutData( gridData4 );
      removeParamButton= new Button( paramsComposite, SWT.NONE );
      removeParamButton.setText( "Remove" );
      removeParamButton.setLayoutData( gridData1 );
   }
   
   /**
    * This method initializes attrsComposite
    * 
    */
   private void createAttrsComposite() {
      GridData gridData5= new GridData();
      gridData5.widthHint= 50;
      GridData gridData3= new GridData();
      gridData3.widthHint= 50;
      GridData gridData2= new GridData();
      gridData2.widthHint= 50;
      attrsComposite= new Composite( rightGroup, SWT.NONE );
      attrsComposite.setLayout( new GridLayout() );
      addAttrButton= new Button( attrsComposite, SWT.NONE );
      addAttrButton.setText( "Add" );
      addAttrButton.setLayoutData( gridData2 );
      editAttrButton= new Button( attrsComposite, SWT.NONE );
      editAttrButton.setText( "Edit" );
      editAttrButton.setLayoutData( gridData5 );
      removeAttrButton= new Button( attrsComposite, SWT.NONE );
      removeAttrButton.setText( "Remove" );
      removeAttrButton.setLayoutData( gridData3 );
   }
   
}
