package com.unitesk.requality.eclipse.editors.browser;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.handlers.IHandlerService;
import org.eclipse.ui.views.contentoutline.ContentOutlinePage;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertySheetPageContributor;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;

import com.unitesk.requality.eclipse.views.CNFLabelProvider;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;

public class ReqMarkerOutlinePage extends ContentOutlinePage implements
        ITabbedPropertySheetPageContributor, IAdaptable {
    private Document input;
    private ISelectionListener locationSelectionListener = new ISelectionListener() {
        @Override
        public void selectionChanged(IWorkbenchPart part, ISelection selection) {
            if (!(selection instanceof IStructuredSelection)) {
                return;
            }
            if (getTreeViewer().getSelection().equals(selection)) {
                return;
            }
            IStructuredSelection sel = (IStructuredSelection) selection;
            if (!(sel.getFirstElement() instanceof Location)) {
                return;
            }
            Location loc = (Location) sel.getFirstElement();
            getTreeViewer().setSelection(new StructuredSelection(loc));
        }
    };

    private IDoubleClickListener doubleClickListener = new IDoubleClickListener() {
        @Override
        public void doubleClick(DoubleClickEvent event) {
            IHandlerService handlerService = (IHandlerService) PlatformUI
                    .getWorkbench().getActiveWorkbenchWindow().getActivePage()
                    .getActivePart().getSite()
                    .getService(IHandlerService.class);
            try {
                handlerService.executeCommand(
                        "com.unitesk.requality.commands.OpenNode", null);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    };

    @Override
    public void createControl(Composite parent) {
        super.createControl(parent);
        getTreeViewer().setContentProvider(
                new ReqMarkerOutlineContentProvider());
        getTreeViewer().setLabelProvider(new CNFLabelProvider());
        getTreeViewer().setInput(input);
        getSite().getWorkbenchWindow().getSelectionService()
                .addPostSelectionListener(locationSelectionListener);
        getTreeViewer().addDoubleClickListener(doubleClickListener);
    }

    public void setInput(Object input) {
        this.input = (Document) input;
    }

    public void refresh() {
        getSite().getShell().getDisplay().asyncExec(new Runnable() {
            @Override
            public void run() {
                if(!getTreeViewer().getTree().isDisposed()) {
                    getTreeViewer().refresh();
                }
            }
        });
    }

    @Override
    public void dispose() {
        super.dispose();
        getSite().getWorkbenchWindow().getSelectionService()
                .removePostSelectionListener(locationSelectionListener);
    }

    @Override
    public String getContributorId() {
        return "com.unitesk.requality.views.RequirementPropertyContributor";
    }

    @SuppressWarnings("rawtypes")
    @Override
    public Object getAdapter(Class required) {
//        if (required == IContributedContentsView.class) {
//            return new IContributedContentsView() {
//                public IWorkbenchPart getContributingPart() {
//                    return PlatformUI
//                            .getWorkbench()
//                            .getActiveWorkbenchWindow()
//                            .getActivePage()
//                            .findEditor(input);
//                }
//            };
//        }
        if (required == IPropertySheetPage.class) {
            return new TabbedPropertySheetPage(this);
        }
        return Platform.getAdapterManager().getAdapter(this, required);
    }
}
