package com.unitesk.requality.eclipse.handlers.document;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.views.RequalityCNF;
import com.unitesk.requality.nodetypes.DocFolder;
import com.unitesk.requality.nodetypes.Document;

public class CreateDocFolderHandler extends AbstractHandler {

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        TreeNode node = RequalityCNF.getSelectedNode();
        final DocFolder parent;
        if (node instanceof Document) {
            parent = (DocFolder) node.getParent();
        } else if (node instanceof DocFolder) {
            parent = (DocFolder) node;
        } else {
            throw new UnsupportedOperationException("Wrong selection type");
        }

        InputDialog dlg = new InputDialog(PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow().getShell(), "New Document Folder",
                "Enter folder name", "new_folder", new IInputValidator() {
                    @Override
                    public String isValid(String newText) {
                        if (newText.length() == 0) {
                            return "enter folder name";
                        }
                        if (newText.lastIndexOf("/") != -1) {
                            return "folder name can't contains '/'";
                        }
                        if (parent.findChild(newText) != null) {
                            return "child with same id is exist";
                        }
                        return null;
                    }
                });

        if (dlg.open() == Dialog.OK) {
            TreeNode folder = parent.createChild(dlg.getValue(), "DocFolder");
            Activator.getDefault().setActiveNode(folder, "DocFolder");
        }
        return null;
    }
}
