/*
 * Copyright 2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.ui.dialogs;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.ITreeChangeListener;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.tools.DefaultTreeChangeListener;
import com.unitesk.requality.eclipse.tools.TreeDBNodeComparator;
import com.unitesk.requality.eclipse.views.CNFLabelProvider;
import com.unitesk.requality.eclipse.views.TreesContentProvider;
import com.unitesk.requality.nodetypes.Requirement;

// TODO: use CNF with filter (remove dependency on ReqLabelProvider)
abstract public class SelectNodeDialog extends Dialog {

    private final class NodeSelectionListener implements
            ISelectionChangedListener {
        @Override
        public void selectionChanged(SelectionChangedEvent event) {
            TreeSelection ts = (TreeSelection) event.getSelection();
            if (!isSelectedSame(selectedNode)) {
                if (ts.getFirstElement() instanceof TreeNode) {
                    select(ts.getFirstElement());
                } else if (ts.getFirstElement() instanceof TreeDB) {
                    TreeDB db = (TreeDB) ts.getFirstElement();
                    String rootQId = ((TreesContentProvider)nodeTree.getContentProvider()).getRootQId();
                    select(db.getNode(rootQId));
                }
                updateUI();
            }
        }
    }

    private ITreeChangeListener catalogChangeListener = new DefaultTreeChangeListener() {

        @Override
        public void changed(TreeNode node) {
            Shell shell = getShell();
            if (shell == null)
                return;

            Display display = shell.getDisplay();
            if (display.isDisposed())
                return;

            display.asyncExec(new Runnable() {
                @Override
                public void run() {
                    refresh();
                }
            });
        }
    };

    private MouseListener mouseList = new MouseListener() {
        public void mouseDoubleClick(MouseEvent e) {
            TreeItem item = nodeTree.getTree().getItem(new Point(e.x, e.y));
            if (item != null && canSelectRequirement())
                okPressed();
        }

        public void mouseDown(MouseEvent e) {
        }

        public void mouseUp(MouseEvent e) {
        }
    };

    private KeyListener keyList = new KeyListener() {
        public void keyPressed(KeyEvent e) {
            if (e.keyCode == ' ') {
                TreeItem[] items = nodeTree.getTree().getSelection();
                if (items.length > 0) {
                    TreeItem item = items[0];
                    if (item.getExpanded() == false) {
                        item.setExpanded(true);
                        Event e1 = new Event();
                        e1.item = item;
                        nodeTree.getTree().notifyListeners(SWT.Expand, e1);
                    } else
                        item.setExpanded(false);
                }
            }
        }

        public void keyReleased(KeyEvent e) {
        }
    };

    private static final String TEXT_CREATE_REQUIREMENT = "create new";

    private static int TREE_WIDTH = 250;
    private static int TREE_HEIGHT = 300;

    public String getTitle() {
        return "Node selection";
    }

    private TreeDB db = null;

    private boolean canSelectRoot = false;

    private Button createButton = null;
    protected TreeNode selectedNode = null;

    private TreeViewer nodeTree = null;

    private final SelectionListener createButtonListener = new SelectionListener() {
        @Override
        public void widgetSelected(SelectionEvent e) {
            EditNodeIdDialog dialog = new EditNodeIdDialog(SelectNodeDialog.this
                    .getShell(), selectedNode, null);
            if ((dialog.open() == EditNodeIdDialog.OK)
                    && (selectedNode.getTreeDB() != null)) {

                TreeNode newReq = selectedNode.createChild(dialog.getResult());
                newReq.saveAttributes();
                select(newReq);
//                ReqTreeView.setSelected((TreeNode) selectedNode);
            }
        }

        @Override
        public void widgetDefaultSelected(SelectionEvent e) {
            widgetDefaultSelected(e);
        }
    };

    @Override
    public int open() {
        db.addListener(catalogChangeListener);
        return super.open();
    }

    @Override
    protected void buttonPressed(int buttonId) {
        super.buttonPressed(buttonId);
        db.removeListener(catalogChangeListener);
    }

    public boolean canCreateNode() {
            return selectedNode != null;
    }

    public boolean canSelect() {
        return selectedNode != null && canSelectRequirement();
    }

    private void updateUI() {
        Button ok = getButton(IDialogConstants.OK_ID);
        if(ok == null ||ok.isDisposed()) {
            return;
        }
        if (createButton != null) {
            createButton.setEnabled(canCreateNode());
        }
        ok.setEnabled(canSelect());
    }

    private boolean canSelectRequirement() {
        if (selectedNode == null)
            return false;

        if (canSelectRoot)
            return true; // we already select something
        String rootQId = ((TreesContentProvider)nodeTree.getContentProvider()).getRootQId();
        boolean result = !rootQId.equals(selectedNode.getQualifiedId());
        return result;
    }

    public SelectNodeDialog(Shell shell, TreeDB treedb) {
        super(shell);
        this.db = treedb;
    }

    public TreeNode getResult() {
        if (selectedNode instanceof TreeNode) {
            return selectedNode;
        }
        return null;
    }

    @Override
    protected Control createDialogArea(Composite parent) {
        Composite container = createClientContainer(parent, 1);

        Composite header = new Composite(container, SWT.NONE);
        header.setLayout(new GridLayout(2, false));
        Label commentLabel = new Label(header, SWT.NONE);
        String labelString = "Select a node";
        if(canCreateNodes()) {
            labelString+=" or ";
            createButton = new Button(header, SWT.PUSH);
            createButton.setText(TEXT_CREATE_REQUIREMENT);
            createButton.addSelectionListener(createButtonListener);
        }
        commentLabel.setText(labelString);
        commentLabel.setLayoutData(new GridData(GridData.BEGINNING,
                GridData.CENTER, true, true));
        nodeTree = new TreeViewer(container);
        nodeTree.getControl().setLayoutData(
                new GridData(TREE_WIDTH, TREE_HEIGHT));
        nodeTree.setLabelProvider(new CNFLabelProvider(){
            @Override
            public String getText(Object element) {
                if (element instanceof Requirement) {
                    Requirement node = (Requirement) element;
                    if (node.equals(node.getTreeDB().getNode(
                            Requirement.getTypeRootQId())))
                        return getText(node.getTreeDB());
                    
                }
                return super.getText(element);
            }
            @Override
            public Image getImage(Object element) {
                if (element instanceof TreeNode) {
                    TreeNode node = (TreeNode) element;
                    if (node.getQualifiedId().equals(
                            node.getTreeDB().getNode(Requirement.getTypeRootQId())
                                    .getQualifiedId()))
                        return getImage(node.getTreeDB());

                    return Activator.getDefault().getImageRegistry().get("nodetype."+node.getType());
                }
                return super.getImage(element);
            }
        });
        nodeTree.setSorter(new ViewerSorter());
        nodeTree.setAutoExpandLevel(2);
        nodeTree.setComparator(new ViewerComparator(new TreeDBNodeComparator()));
        if (db == null) {
            nodeTree.setContentProvider(getContentProvider());
            nodeTree.setInput(ResourcesPlugin.getWorkspace().getRoot());
        } else {
            nodeTree.setContentProvider(getContentProvider());
            nodeTree.setInput(db);
        }

        nodeTree.addSelectionChangedListener(new NodeSelectionListener());

        nodeTree.getTree().addMouseListener(mouseList);
        nodeTree.getTree().addKeyListener(keyList);

        this.getShell().setText(getTitle());
        this.getShell().setImage(
                PlatformUI.getWorkbench().getSharedImages().getImage(
                        ISharedImages.IMG_OBJ_ELEMENT));

        refresh();

        return container;
    }

    public boolean canCreateNodes() {
        return false;
    }

    protected TreesContentProvider getContentProvider() {
        return new TreesContentProvider("/");
    }

    protected Composite createClientContainer(Composite parent, int span) {
        Composite container = new Composite(parent, SWT.NONE);
        GridLayout layout = new GridLayout();
        layout.marginWidth = layout.marginHeight = 10;
        layout.verticalSpacing = 2;
        layout.numColumns = span;
        container.setLayout(layout);
        return container;
    }

    @Override
    protected void createButtonsForButtonBar(Composite parent) {
        super.createButtonsForButtonBar(parent);
        updateUI();
    }

    public void setCanSelectRoot(boolean canSelectRoot) {
        this.canSelectRoot = canSelectRoot;
    }

    public void select(Object obj) {
        if (obj != null) {
            selectedNode = (TreeNode) obj;
            if (nodeTree != null && !nodeTree.getControl().isDisposed()) {
                nodeTree.expandToLevel(selectedNode, 0);
                nodeTree.setSelection(new StructuredSelection(obj), true);
                updateUI();
            }
        }
    }

    private boolean isSelectedSame(TreeNode selectedReq) {
        IStructuredSelection selection = (IStructuredSelection) nodeTree
                .getSelection();
        if (selection.getFirstElement() == null)
            return false;
        if (!selection.getFirstElement().equals(selectedReq))
            return false;
        return true;
    }

    private void refresh() {
        if (nodeTree != null && !nodeTree.getControl().isDisposed()) {
            nodeTree.refresh();

            if (selectedNode != null) {
                // couse base can be reloaded
                selectedNode = selectedNode.getTreeDB().getNode(
                        selectedNode.getUUId());
                if (!isSelectedSame(selectedNode)) {
                    select(selectedNode);
                }
            }
        }
    }
}
