package com.unitesk.requality.eclipse.views.actions.documents;

import org.eclipse.core.resources.IResource;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.DocumentProcessorDescriptor;
import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.eclipse.editors.browser.ReqMarker;
import com.unitesk.requality.eclipse.tools.ResourceStorage;
import com.unitesk.requality.eclipse.views.actions.CNFAction;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Requirement;

public class DocumentProcessorAction extends CNFAction {
    private DocumentProcessorDescriptor desc;

    public DocumentProcessorAction(DocumentProcessorDescriptor desc,
            ISelectionProvider provider) {
        // TODO: 
//        super(provider);
        this.desc = desc;
        setText(desc.getName());
    }

    @Override
    public void run() {
        TreeDB db = (TreeDB) getTreeDB();
        Document doc = getSelectedDocument();
        Requirement rootReq = (Requirement) db.getNode(Requirement.getTypeRootQId());
        try {
            boolean closed = closeEditor(doc);
            if(! desc.getProcessor().configure(doc)) {
                return;
            }
            desc.getProcessor().process(doc.getQualifiedId(), rootReq);
            ResourceStorage rs = (ResourceStorage)db.getStorage();
            rs.getResource(rootReq.getUUId()).refreshLocal(
                    IResource.DEPTH_INFINITE, null);
            if (closed) {
                IWorkbenchPage page = PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getActivePage();
                if (page != null) {
                    page.openEditor(new NodeEditorInput(doc),
                            "com.unitesk.requality.editors.browser.ReqMarker",
                            true);
                }
            }
        } catch (Exception e) {
            MessageDialog.openError(PlatformUI.getWorkbench().getDisplay()
                    .getActiveShell(), "Document Processing Error",
                    e.getClass().getName()+": "+e.getMessage());
            e.printStackTrace();
        }
    }

    private boolean closeEditor(Document doc) {
        try {
            IEditorReference eRef = Activator.findMarkEditorForDocument(doc
                    .getQualifiedId());
            if (eRef == null)
                return false;
            ((ReqMarker) eRef.getEditor(false)).close(false);
            return true;
        } catch (PartInitException e) {
            e.printStackTrace();
        }
        return false;
    }

    private TreeDB getTreeDB() {
        TreeNode node = getSelectedNode();

        if (node !=null ) {
            return node.getTreeDB();
        }
        return null;
    }

    @Override
    public boolean isEnabled() {
        return getSelectedDocument() != null;
    }

    private Document getSelectedDocument() {
        TreeNode node = getSelectedNode();
        if (node instanceof Document) {
            return (Document) node;
        }
        return null;
    }
}
