package com.unitesk.requality.eclipse.views.properties;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.TextViewer;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.eclipse.editors.browser.ReqMarker;
import com.unitesk.requality.eclipse.views.CNFLabelProvider;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;

public class AltDescriptionPropertySection extends RequirementPropertiesTab {

    private TextViewer adesc;
    private TreeViewer locations;

    public AltDescriptionPropertySection() {
    }

    @Override
    public void createControls(Composite parent,
            TabbedPropertySheetPage aTabbedPropertySheetPage) {
        super.createControls(parent, aTabbedPropertySheetPage);
        TabbedPropertySheetWidgetFactory wf = aTabbedPropertySheetPage
                .getWidgetFactory();
        parent.setLayout(new GridLayout());
        parent.setLayoutData(new GridData(GridData.FILL_BOTH));
        wf.createLabel(parent, "Alternative Description:");
        adesc = new TextViewer(parent, SWT.MULTI | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
        GridData gridData = new GridData(GridData.FILL_BOTH);
        gridData.widthHint = 0;
        gridData.heightHint = 0;
        adesc.getControl().setLayoutData(gridData);
        adesc.getControl().addFocusListener(new FocusAdapter() {
            public void focusLost(FocusEvent e) {
                try {
                    req.setDescription(adesc.getDocument().get(0,
                            adesc.getDocument().getLength()));
                } catch (BadLocationException ex) {
                    ex.printStackTrace();
                }
                req.saveAttributes();
            }
        });

        wf.createLabel(parent, "Locations:");
        locations = new TreeViewer(parent);
        gridData = new GridData(GridData.FILL_BOTH);
        gridData.widthHint = 0;
        gridData.heightHint = 0;
        locations.getControl().setLayoutData(gridData);
        locations.setLabelProvider(new CNFLabelProvider());
        locations.setSorter(new ViewerSorter());
        locations.setContentProvider(new ITreeContentProvider() {
            @Override
            public void inputChanged(Viewer viewer, Object oldInput,
                    Object newInput) {
            }

            @Override
            public void dispose() {
            }

            @Override
            public boolean hasChildren(Object element) {
                TreeNode node = (TreeNode) element;
                return node.sizeChildren() > 0;
            }

            @Override
            public Object getParent(Object element) {
                return ((TreeNode) element).getParent();
            }

            @Override
            public Object[] getElements(Object inputElement) {
                return (Object[]) inputElement;
            }

            @Override
            public Object[] getChildren(Object parentElement) {
                TreeNode node = (TreeNode) parentElement;
                ArrayList<Object> result = new ArrayList<Object>();
                if (node.getType().equals("Document")) {
                    for (int i = 0; i < node.sizeChildren(); i++) {
                        Location loc = (Location) node.getChild(i);
                        if (Arrays.asList(loc.getRequirements()).contains(req)) {
                            result.add(loc);
                        }
                    }
                }
                return result.toArray();
            }
        });
        locations.addDoubleClickListener(new IDoubleClickListener() {

            @Override
            public void doubleClick(DoubleClickEvent event) {
                IStructuredSelection selection = (IStructuredSelection) locations
                        .getSelection();
                NodeEditorInput input;
                if (selection.getFirstElement() instanceof Document) {
                    Document doc = (Document) selection.getFirstElement();
                    input = new NodeEditorInput(doc);
                } else if (selection.getFirstElement() instanceof Location) {
                    Location loc = (Location) selection.getFirstElement();
                    input = new NodeEditorInput(loc.getParent());
                    input.setData(loc);
                } else {
                    return;
                }

                try {
                    String docId = input.getStorage().getNode()
                            .getQualifiedId();
                    IEditorReference ref = Activator
                            .findMarkEditorForDocument(docId);
                    if (ref != null && ref.getEditor(true) != null) {
                        ReqMarker ed = (ReqMarker) ref.getEditor(true);
                        ed.setTargetLocation((Location) input.getData());
                    }
                    IDE.openEditor(PlatformUI.getWorkbench()
                            .getActiveWorkbenchWindow().getActivePage(), input,
                            ReqMarker.ID);
                } catch (PartInitException e) {
                    e.printStackTrace();
                } catch (CoreException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }
        });
    }

    @Override
    public void dispose() {
        super.dispose();
    }

    @Override
    protected void setRequirement(Requirement req) {
        if(locations==null || locations.getControl().isDisposed()) {
            return;
        }
        adesc.setDocument(new org.eclipse.jface.text.Document(req
                .getDescription()));
        Set<com.unitesk.requality.nodetypes.Document> docs = new HashSet<com.unitesk.requality.nodetypes.Document>();
        for (Location l : req.getLocations()) {
            if (!docs.contains(l.getParent())) {
                docs.add((com.unitesk.requality.nodetypes.Document) l
                        .getParent());
            }
        }
        locations.setInput(docs.toArray());
        locations.expandAll();
    }
}
