package com.unitesk.requality.eclipse.views.properties;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.views.properties.tabbed.AbstractPropertySection;

import com.unitesk.requality.core.ITreeChangeListener;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.tools.ITreeStateListener;
import com.unitesk.requality.eclipse.tools.TreesTracker;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;

public abstract class RequirementPropertiesTab extends AbstractPropertySection {
    protected Requirement req;

    ITreeChangeListener treeChangeListener = new ITreeChangeListener() {
        @Override
        public void moved(TreeNode node, TreeNode oldParent) {
            if (req != null)
                setRequirementSafe(req);
        }

        @Override
        public void deleted(TreeNode node) {
            if (req != null)
                setRequirementSafe(req);
        }

        @Override
        public void changed(TreeNode node) {
            if (req != null)
                setRequirementSafe(req);
        }

        @Override
        public void added(TreeNode node) {
            if (req != null)
                setRequirementSafe(req);
        }
    };

    ITreeStateListener treeStateListener = new ITreeStateListener() {
        @Override
        public void open(TreeDB db) {
            db.addListener(treeChangeListener);
            if (req != null)
                setRequirementSafe(req);
        }

        @Override
        public void close(TreeDB db) {
            db.removeListener(treeChangeListener);
            if (req != null)
                setRequirementSafe(req);
        }
    };

    public RequirementPropertiesTab() {
        super();
        for (TreeDB db : TreesTracker.getTracker().getTrees()) {
            db.addListener(treeChangeListener);
        }
        TreesTracker.getTracker().addListener(treeStateListener);
    }

    @Override
    public void dispose() {
        for (TreeDB db : TreesTracker.getTracker().getTrees()) {
            db.removeListener(treeChangeListener);
        }
        TreesTracker.getTracker().removeListener(treeStateListener);
        super.dispose();
    }

    private void setRequirementSafe(final Requirement req) {
        getPart().getSite().getShell().getDisplay().syncExec(new Runnable() {
            @Override
            public void run() {
                setRequirement(req);
            }
        });
    }

     abstract protected void setRequirement(Requirement req);

    @Override
    public void setInput(IWorkbenchPart part, ISelection selection) {
        super.setInput(part, selection);
        req = RequirementPropertiesTab.toRequirement(selection);
        if (req != null) {
            setRequirementSafe(req);
        } else {
            // Unsupported input
        }
    }

    public static Requirement toRequirement(ISelection sel) {
        TreeNode node = (TreeNode) ((IStructuredSelection) sel)
                .getFirstElement();
        if (node instanceof Requirement) {
            return (Requirement) node;
        }
        if (node instanceof Location) {
            Location location = (Location) node;
            Requirement[] reqs = location.getRequirements();
            if (reqs.length > 0) {
                return reqs[0];
            }

        }

        return null;
    }

}
