/*
 * Copyright 2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.tools;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.nio.channels.FileChannel;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import com.unitesk.requality.core.FSStorage;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.instancer.TreeInstancer;
import com.unitesk.requality.nodetypes.DocFolder;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.report.ReportGenerator;
import com.unitesk.requality.report.Template;

//
public class RequalityCLI {

    public final static String cmdName = "requality";
    private static Map<String, String> usages = new HashMap<String, String>();

    static {

        usages.put(
                "report",
                "Usage: "
                        + cmdName
                        + " report "
                        + "<path to template> <path to catalogue> <save location> [<root qid>]");
        usages.put(
                "convert",
                "Usage: "
                        + cmdName
                        + " convert "
                        + "<path to database> <path to document> [<document folder qid>] [<group requirement qid>]");
    }

    public static void usage() {
        log("Usage: " + cmdName + " " + "<help|convert|report> [params]");
    }

    public static void usageHelp(String param) {
        StringBuilder result = new StringBuilder();

        if (usages.containsKey(param)) {
            result.append(usages.get(param));
        } else {
            usage();
        }

        log(result.toString());
    }

    private static void log(String string) {
        System.out.println(string);
    }

    public static void main(String[] args) throws Exception {
        if (args.length == 0) {
            usage();
            return;
        }
        System.out
                .println("Welcome to Requality console mode! Process started...");

        if ("help".equals(args[0])) {
            String subj = args.length > 1 ? args[1] : "";
            usageHelp(subj);
            return;
        }
        // [<document folder qid>] [<group requirement qid>]
        if ("convert".equals(args[0])) {
            if (args.length > 2 && args.length < 6) {
                String pathToDB = args[1];
                String pathToDoc = args[2];
                String[] qids = new String[args.length - 3];
                for (int i = 3; i < args.length; i++) {
                    qids[i - 3] = args[i];
                }

                File inFile = new File(pathToDoc);
                TreeDB db = new TreeDB(new FSStorage(pathToDB));
                db.addNodeType("Requirement", Requirement.class);
                db.addNodeType("Location", Location.class);
                db.addNodeType("DocFolder", DocFolder.class);
                db.addNodeType("Document", Document.class);
                Requirement groupNode = (Requirement) db.getNode(Requirement
                        .getTypeRootQId());
                DocFolder folderNode = (DocFolder) db.getNode(Document
                        .getTypeRootQId());

                Document doc = (Document) folderNode.createChild(
                        inFile.getName(), "Document");
                Converter c = new Converter();
                try {
                    Writer out = new FileWriter(doc.getHTMLPath());
                    Map<String, String[]> result = c.convert(new FileReader(
                            inFile), out);
                    c.store(result, doc.getQualifiedId(), db, groupNode);
                    System.out.println("OK");
                } catch (IOException e) {
                    System.out.println("ERROR");
                    e.printStackTrace();
                }
                return;
            }
            usageHelp("convert");
            return;
        }

        if ("report".equals(args[0])) {
            if (!(args.length >= 4 && args.length <= 5)) {
                usageHelp("report");
                return;
            }
            String reportArgs[] = Arrays.copyOfRange(args, 1, args.length);
            TreeDB db = new TreeDB(new FSStorage(reportArgs[1]));
            db.addNodeType("Requirement", Requirement.class);
            db.addNodeType("Location", Location.class);
            db.addNodeType("DocFolder", DocFolder.class);
            db.addNodeType("Document", Document.class);
            
            String templatePath = reportArgs[0];
            Template tpl = new Template(templatePath);
            String rootQId = Requirement.getTypeRootQId();
            String saveTo = reportArgs[2];
            if (reportArgs.length > 3) {
                rootQId = reportArgs[3];
            }
            TreeDB instTree = new TreeDB(new TreeInstancer(db));
            instTree.addNodeType("Requirement", Requirement.class);
            instTree.addNodeType("Location", Location.class);
            instTree.addNodeType("DocFolder", DocFolder.class);
            instTree.addNodeType("Document", Document.class);
            
            Requirement rootReq = (Requirement) instTree.getNode(rootQId);
            ReportGenerator.genetareReport(rootReq, tpl, saveTo);
            return;
        }
//        if ("docclean".equals(args[0])) {
//            if (args.length == 2) {
//                docclean(args[1]);
//            } else {
//                System.out.println("neep path to database");
//            }
//            return;
//        }

        usage();
    }

    // private static void docclean(String path) {
    // TreeDB db = new TreeDBIO(path);
    // Map<String, Set<UUID>> docToUUIDs = new HashMap<String, Set<UUID>>();
    // for (TreeDBNode r : new DeepFirstTreeWalker(db.getRootNode(
    // Requirement.type, false))) {
    // Requirement req = (Requirement) r;
    // for (Location loc : req.getLocations()) {
    // String docId = loc.getDocId().replace('\\', '/');
    // if (docToUUIDs.get(docId) == null) {
    //
    // docToUUIDs.put(docId, new HashSet<UUID>());
    // }
    // docToUUIDs.get(docId).add(loc.getId());
    // }
    // }
    // for (String docId : docToUUIDs.keySet()) {
    // Set<UUID> locRemove = new HashSet<UUID>();
    // Document doc = db.getDocument(docId);
    // System.out.println("processing document: " + docId + " locations ("
    // + docToUUIDs.get(docId).size() + "/" + doc.sizeLocations()
    // + ")");
    //
    // for (UUID locId : doc.getLocations()) {
    // locRemove.add(locId);
    // }
    // locRemove.removeAll(docToUUIDs.get(docId));
    // for (UUID locId : locRemove) {
    // System.out.println("Remove location " + locId.toString()
    // + " from " + docId);
    // doc.removeLocation(locId);
    // }
    // }
    // }

    public static void copyfile(String srFile, String dtFile)
            throws IOException {
        File sourceFile = new File(srFile);
        File destFile = new File(dtFile);
        if (!destFile.exists()) {
            destFile.createNewFile();
        }
        FileChannel source = null;
        FileChannel destination = null;
        try {
            source = new FileInputStream(sourceFile).getChannel();
            destination = new FileOutputStream(destFile).getChannel();
            destination.transferFrom(source, 0, source.size());
        } finally {
            if (source != null) {
                source.close();
            }
            if (destination != null) {
                destination.close();
            }
        }
    }

    public static String htmlEscape(String str) {
        return str.replace("&", "&amp;").replace("<", "&lt;")
                .replace(">", "&gt;");
    }
}
